

######################################################################
#                                                                    #
# Supplement                                                         #
#                                                                    #
# Keil P. & Jetz W.: Downscaling the environmental associations      #
# and spatial patterns of species richness. Ecological Applications. #
#                                                                    #
# Created on 10/16/2013 1:59:29 PM by Petr Keil                      #
# E-mail: pkeil@seznam.cz                                            #
#                                                                    #
# ------------------------------------------------------------------ #
#                                                                    #
# Description:                                                       #
# Here we provide the R and JAGS codes for the Reference model,      #
# Downscaling model 1 and Downscaling model 2. If you want to run it #
# make sure to exectue the commands in the DATA PREPARATION section. #
# ------------------------------------------------------------------ #
#                                                                    #
# The analysis was performed using the following                     #
# software and packages:                                             #
#   JAGS 3.3.0                                                       #
#   R 2.15.0                                                         #
#   R library rjags                                                  #
#                                                                    #
######################################################################




######################################################################
#                                                                    #
#  DATA PREPARATION                                                  #
#                                                                    #
######################################################################

# Loading the fine-grain data:
  fine.data <- read.csv("fine_data.csv")
# Loading the coarse-grain data:
  coarse.data <- read.csv("coarse_data.csv")
  # NOTE: Use setwd() to point R to the directory that contains
  # the two data files!


# Standardizing and centering the fine-grain environmental predictors:
  fine.data[,3:8] <- scale(fine.data[,3:8])
# Rounding the scaled variables to 3 decimal places (to speed-up
# the MCMC and to save some memory):
  fine.data[,3:8] <- round(fine.data[,3:8], 3)


# Data from the 600 well-surveyed grid cells of the Reference dataset:
  ref.data <- fine.data[fine.data$ref.data == 1, ]




######################################################################
#                                                                    #
#  REFERENCE MODEL                                                   #
#                                                                    #
######################################################################

# Putting the data together for JAGS:
  ref.data.jags <- list(S.ref = ref.data$S.fine,
                        N.ref = nrow(ref.data),
                        NPP.ref = ref.data$NPP,
                        LC.ref = ref.data$LC,
                        PW.ref = ref.data$PW,
                        HFP.ref = ref.data$HFP,
                        PS.ref = ref.data$PS,
                        T.ref = ref.data$T,
                        N.fine = nrow(fine.data),
                        NPP = fine.data$NPP,
                        LC = fine.data$LC,
                        PW = fine.data$PW,
                        HFP = fine.data$HFP,
                        PS = fine.data$PS,
                        T = fine.data$T)

# Loading the JAGS library:
  library(rjags)

# Model definition for JAGS:
  cat("
    model
    {
      # priors
      beta0 ~ dnorm(0, 0.01)
      beta1 ~ dnorm(0, 0.1)
      beta2 ~ dnorm(0, 0.1)
      beta3 ~ dnorm(0, 0.1)
      beta4 ~ dnorm(0, 0.1)
      beta5 ~ dnorm(0, 0.1)
      beta6 ~ dnorm(0, 0.1)

      # fitting the model to the fine-grain reference dataset
      # (600 well surveyed cells)
        for (j in 1:N.ref)
        {
          # Eq. 1 (see Methods):
          log(lambda.fine[j]) <- beta0 +
                                 beta1 * NPP.ref[j] +
                                 beta2 * LC.ref[j] +
                                 beta3 * PW.ref[j] +
                                 beta4 * HFP.ref[j] +
                                 beta5 * PS.ref[j] +
                                 beta6 * T.ref[j]
          # Eq. 2 (see Methods):
          S.ref[j] ~ dpois(mu.fine[j])
        }

      # predicting in the complete fine-grain dataset
      # (all 6238 fine-grain grid cells)
        for (i in 1:N.fine)
        {
          log(pred.fine[i]) <- beta0 +
                               beta1 * NPP[i] +
                               beta2 * LC[i] +
                               beta3 * PW[i] +
                               beta4 * HFP[i] +
                               beta5 * PS[i] +
                               beta6 * T[i]
        }
    }
  ", file="ref_model.txt")

# Function that generates the initial values for MCMC:
  inits <- function()
  {
     list(beta0=rnorm(1),
          beta1=rnorm(1),
          beta2=rnorm(1),
          beta3=rnorm(1),
          beta4=rnorm(1),
          beta5=rnorm(1),
          beta6=rnorm(1))
  }
  inits3 <- list(inits(), inits(), inits())

# Parameters that will be monitored:
  params <- c("beta0", # intercept
              "beta1", "beta2", "beta3", # slopes
              "beta4", "beta5", "beta6",
              "pred.fine") # fine-grain predictions

# WARNING: It may take a long time to fit the model with the current
# settings. If you want a faster (and less accurate) fit, reduce the
# n.iter to 10,000 and n.chains to 1.

# Model compilation:
  jm <- jags.model("ref_model.txt",
                   data = ref.data.jags,
                   inits = inits3,
                   n.chains = 3,
                   n.adapt = 1000)

# Burn-in phase:
  update(jm, n.iter = 100000)

# Sampling:
  jm.sample.ref <- jags.samples(jm,
                                variable.names = params,
                                n.iter = 100000,
                               thin = 100)

# Example of one parameter summary - beta1:
  one.parameter <- as.mcmc.list(jm.sample.ref$beta1)
  plot(one.parameter)
  summary(one.parameter)



######################################################################
#                                                                    #
#  DOWNSCALING MODEL 1                                               #
#                                                                    #
######################################################################

# Putting the data together for JAGS:
  downscaling.data.jags <- list(NPP = fine.data$NPP,
                                LC = fine.data$LC,
                                PW = fine.data$PW,
                                HFP = fine.data$HFP,
                                PS = fine.data$PS,
                                T = fine.data$T,
                                N.coarse = nrow(coarse.data),
                                S.coarse = coarse.data$S.coarse,
                                START = coarse.data$START,
                                STOP = coarse.data$STOP)

# Loading the JAGS library:
  library(rjags)

# Model definition for JAGS:
  cat("
    model
    {
      # priors
      beta0 ~ dnorm(0, 0.01)
      beta1 ~ dnorm(0, 0.1)
      beta2 ~ dnorm(0, 0.1)
      beta3 ~ dnorm(0, 0.1)
      beta4 ~ dnorm(0, 0.1)
      beta5 ~ dnorm(0, 0.1)
      beta6 ~ dnorm(0, 0.1)

      # strong informative prior on betadiversity (Beta_w)
      betaw <- 1.8

      # iterating through coarse-grain cells
      for(i in 1:N.coarse)
      {
        # The species-area relationship,
        # Eq. 3 (see Methods):
        lambda.coarse[i] <-betaw * mean(lambda.fine[START[i]:STOP[i]])
        # Eq. 4 (see Methods):
        S.coarse[i] ~ dpois(lambda.coarse[i])

        # iterating through fine-scale cells
        for (j in START[i]:STOP[i])
        {
          # Eq. 1 (see Methods):
          log(lambda.fine[j]) <- beta0 +
                                 beta1 * NPP[j] +
                                 beta2 * LC[j] +
                                 beta3 * PW[j] +
                                 beta4 * HFP[j] +
                                 beta5 * PS[j] +
                                 beta6 * T[j]
        }
      }
    }
    ", file="downscaling_model1.txt")

# Function that generates the initial values for MCMC:
  inits <- function()
  {
     list(beta0=rnorm(1),
          beta1=rnorm(1),
          beta2=rnorm(1),
          beta3=rnorm(1),
          beta4=rnorm(1),
          beta5=rnorm(1),
          beta6=rnorm(1))
  }
  inits3 <- list(inits(), inits(), inits())

# Parameters that will be monitored:
  params <- c("beta0", # intercept
              "beta1", "beta2", "beta3", # slopes
              "beta4", "beta5", "beta6",
              "lambda.fine") # fine-grain predictions

# WARNING: It may take a long time to fit the model with the current
# settings. If you want a faster (and less accurate) fit, reduce the
# n.iter to 10,000 and n.chains to 1.

# Model compilation:
  jm <- jags.model("downscaling_model1.txt",
                   data = downscaling.data.jags,
                   inits = inits3,
                   n.chains = 3,
                   n.adapt = 1000)

# Burn-in:
  update(jm, n.iter=100000)

# Sampling:
  jm.sample.1 <- jags.samples(jm,
                              variable.names = params,
                              n.iter = 100000,
                              thin = 100)

# Example of one parameter summary - beta1:
  one.parameter <- as.mcmc.list(jm.sample.1$beta1)
  plot(one.parameter)
  summary(one.parameter)



######################################################################
#                                                                    #
#  DOWNSCALING MODEL 2                                               #
#                                                                    #
######################################################################

# The model uses the same data as Downscaling model 1

# Loading the JAGS library:
  library(rjags)

# Model definition for JAGS:
  cat("
    model
    {
    # priors
      beta0 ~ dnorm(0, 0.01)
      beta1 ~ dnorm(0, 0.1)
      beta2 ~ dnorm(0, 0.1)
      beta3 ~ dnorm(0, 0.1)
      beta4 ~ dnorm(0, 0.1)
      beta5 ~ dnorm(0, 0.1)
      beta6 ~ dnorm(0, 0.1)

    # non-informative (but restricted) prior
    # on betadiversity (Beta Whittaker)
      betaw ~ dnorm(1,0.01)T(1,5)
    # Note: this is roughly equivalent to
    # betadiversity ~ dunif(1, 5) ...however,
    # we had problems with this
    # formalism when the model was fitted in OpenBUGS.

    # iterating through coarse-scale cells
      for(i in 1:N.coarse)
      {
        # The species-area relationship,
        # Eq. 3 (see Methods):
        lambda.coarse[i] <-betaw * mean(lambda.fine[START[i]:STOP[i]])
        # Eq. 4 (see Methods):
        S.coarse[i] ~ dpois(lambda.coarse[i])

      # iterating through fine-grain cells
        for (j in START[i]:STOP[i])
        {
          # Eq. 1 (see Methods):
          log(lambda.fine[j]) <- beta0 + beta1 * NPP[j] +
                                     beta2 * LC[j] +
                                     beta3 * PW[j] +
                                     beta4 * HFP[j] +
                                     beta5 * PS[j] +
                                     beta6 * T[j]
        }
      }
    }
  ", file="downscaling_model2.txt")

# Function that generates the initial values for MCMC:
  inits <- function()
  {
     list(beta0=rnorm(1),
          beta1=rnorm(1),
          beta2=rnorm(1),
          beta3=rnorm(1),
          beta4=rnorm(1),
          beta5=rnorm(1),
          beta6=rnorm(1),
          betaw=runif(1,1,5))
  }
  inits3 <- list(inits(), inits(), inits())

# Parameters that will be monitored:
  params <- c("beta0", # intercept
              "beta1", "beta2", "beta3", # slopes
              "beta4", "beta5", "beta6",
              "betaw", # Beta_w
              "lambda.fine") # fine-grain predictions

# WARNING: It may take a long time to fit the model with the current
# settings. If you want a faster (and less accurate) fit, reduce the
# n.iter to 10,000 and n.chains to 1.

# Compiling the model:
  jm <- jags.model("downscaling_model2.txt",
                   data = downscaling.data.jags,
                   inits = inits3,
                   n.chains = 3,
                   n.adapt = 1000)

# Burn-in:
  update(jm, n.iter = 100000)

# Sampling:
  jm.sample.2 <- jags.samples(jm,
                              variable.names = params,
                              n.iter = 100000,
                              thin = 100)

# Example of one parameter summary - beta1:
  one.parameter <- as.mcmc.list(jm.sample.2$beta1)
  plot(one.parameter)
  summary(one.parameter)
